@@ -31,6 +31,11 @@ class WAS (wsgiappservice.WAS):
         except KeyError:
             raise AttributeError ("'was' hasn't attribute '%s'" % name)
 
+    def make_uid (self, s = None):
+        if s is None:
+            s = str (uuid.uuid4 ())
+        return base64.encodebytes (md5 (s.encode ()).digest ()) [:-3].decode ().replace ('/', '-').replace ('+', '.')
+
     # URL builders -------------------------------------------------
     def urlfor (self, thing, *args, **karg):
         # override with resource default args
@@ -73,11 +78,6 @@ class WAS (wsgiappservice.WAS):
         signature_ = base64.b64encode (dig).decode ()
         return signature == signature_
 
-    def make_uid (self, s = None):
-        if s is None:
-            s = str (uuid.uuid4 ())
-        return base64.encodebytes (md5 (s.encode ()).digest ()) [:-3].decode ().replace ('/', '-').replace ('+', '.')
-
     # JWT token --------------------------------------------------
     def encode_jwt (self, claim, salt = None, alg = "HS256"):
         assert "exp" in claim, "exp claim required"
@@ -210,6 +210,9 @@ class WAS (wsgiappservice.WAS):
     def csrf_token_input (self):
         return '<input type="hidden" name="{}" value="{}">'.format (self.CSRF_NAME, self.csrf_token)
 
+    def generate_csrf (self):
+        return self.csrf_token
+
     def remove_csrf (self):
         try:
             del self.cookie [self.CSRF_NAME]
@@ -240,7 +243,9 @@ class WAS (wsgiappservice.WAS):
             ("Cache-Control", "max-age=0"),
             ("Expires", http_date.build_http_date (time.time ()))
         ]
-        if type (headers) is list:
+        if isinstance (headers, dict):
+            headers = list (headers.items ())
+        if headers:
             redirect_headers += headers
         if not body:
             body = self.REDIRECT_TEMPLATE % (status, status, url)
@@ -249,17 +254,17 @@ class WAS (wsgiappservice.WAS):
     def render (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
         return self.app.render (self, template_file, _do_not_use_this_variable_name_, **karg)
 
-    def render_or_API (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
-        if self.request.acceptable ('application/json'):
-            return self.API (None, _do_not_use_this_variable_name_, **karg)
-        return self.render (template_file, _do_not_use_this_variable_name_ = {}, **karg)
-
     def API (self, *args, **karg):
         api = self.response.API (*args, **karg)
         try: api.set_json_encoder (self.app.config.get ("JSON_ENCODER"))
         except AttributeError: pass
         return api
 
+    def render_or_API (self, template_file, _do_not_use_this_variable_name_ = {}, **karg):
+        if self.request.acceptable ('application/json'):
+            return self.API (None, _do_not_use_this_variable_name_, **karg)
+        return self.render (template_file, _do_not_use_this_variable_name_ = {}, **karg)
+
     def Fault (self, *args, **karg):
         fault = self.response.Fault (*args, **karg)
         try: fault.set_json_encoder (self.app.config.get ("JSON_ENCODER"))
@@ -276,7 +281,7 @@ class WAS (wsgiappservice.WAS):
     def Static (self, *args, **karg):
         return self.response.Static (*args, **karg)
 
-    def proxypass (self, alias, path, timeout = 3):
+    def ProxyPass (self, alias, path, timeout = 3):
         def respond (was, rs):
             resp = rs.dispatch ()
             resp.reraise ()
@@ -288,4 +293,4 @@ class WAS (wsgiappservice.WAS):
         headers = dict ([(k, v) for k, v in self.request.headers.items ()])
         headers ["Accept"] = "*/*"
         return self.get ("{}/{}".format (alias, path), headers = headers, timeout = timeout).then (respond)
-    ProxyPass = proxypass
+    proxypass = ProxyPass
